/* globals cgUrl, jQuery, _, Backbone, I18n */
/* jshint undef: true, unused: true, browser: true */
/*
 * Use custom lodash built:
 *     lodash backbone plus=template,mapValues
 */


/**
 * .bind() function polyfill
 *
 * @see https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Function/bind
 */
if (!Function.prototype.bind) {
    Function.prototype.bind = function(oThis) {
        if (typeof this !== "function") {

            // closest thing possible to the ECMAScript 5
            // internal IsCallable function
            throw new TypeError("Function.prototype.bind - what is trying to be bound is not callable");
        }

        var aArgs = Array.prototype.slice.call(arguments, 1),
            fToBind = this,
            FNOP = function() {},
            fBound = function() {
                return fToBind.apply(this instanceof FNOP && oThis ? this : oThis,
                    aArgs.concat(Array.prototype.slice.call(arguments)));
            };

        FNOP.prototype = this.prototype;
        fBound.prototype = new FNOP();

        return fBound;
    };
}


(function($, _, Backbone) {
    'use strict';

    var EMPTY_STRING = '';

    var Q_PARENT = 'p';

    var STYLER_OPTIONS = {
        filePlaceholder: I18n.NoFileSelected,
        fileBrowse: I18n.FileBrowseButton,
        /* disable select search to prevent not localized messages display */
        selectSearch: false,
        selectSearchLimit: 99999,
    };

    var App = {
        root: '/',
        router: null,
        ui: null,

        bootstrap: function() {
            this.router = new AppRouter();
            this.ui = new ViewManager();

            this.ui.showPage(new Page({
                url: window.location.pathname + window.location.search,
                html: getDocumentHTML(document)
            })).then(function() {
                $('body > .logViewInline').remove();
                $('body > .copyright-container').remove();
            });


            Backbone.history.start({
                pushState: true,
                hashChange: false,
                root: this.root
            });

            return this;
        }
    };

    var COMPILED_JST = {},
        render = function(template, data) {
            if (!COMPILED_JST[template]) {
                var $script = $('[id="' + template + '"]');
                if (!$script.length) {
                    throw "No such tmplate " + template;
                }
                COMPILED_JST[template] = _.template($script.text(), false);
            }
            return COMPILED_JST[template].apply(null, [data]);
        };

    (_.templateSettings.imports || (_.templateSettings.imports = {})).render = render;


    var Controller = App.Controller = function Controller(options) {
        options || (options = {}); // jshint ignore:line
        this.initialize.apply(this, arguments);
    };

    _.extend(Controller.prototype, Backbone.Events, {
        // Initialize is an empty function by default. Override it with your own
        // initialization logic.
        initialize: function() {},
    });

    Controller.extend = Backbone.Model.extend;


    var AppRouter = App.Router = Backbone.Router.extend({

        routes: {
            "/*path": "loadPage",
            "*path": "loadPage"
        },

        loadPage: function( /* path, queryArgs */ ) {
            var view = null,
                nav = this.parseModalUrl(window.location.pathname + window.location.search + window.location.hash);

            if (App.ui.mainPage.get('url') !== nav.url) {
                view = App.ui.deferredShowPage(Page.load(nav.url));
            } else {
                view = $.Deferred().resolve().promise();
            }

            if (nav.modalUrl) {
                view.then(function() {
                    return App.ui.deferredShowModalPage(Page.load(nav.modalUrl));
                });
            }

            view.fail(function(error) {
                console.error(error);
                window.location.href = '/';
            });
        },

        getModalUrl: function(pageUrl, modalUrl) {
            if (Backbone.history.history && Backbone.history.history.pushState) {
                return modalUrl + cgUrl.URL_HASH + Q_PARENT + '=' + encodeURIComponent(pageUrl);
            } else {
                return modalUrl;
            }
        },

        parseModalUrl: function(url) {
            var t = url.indexOf(cgUrl.URL_HASH),
                fragment, hash, modalUrl = EMPTY_STRING;

            if (t !== -1) {
                fragment = url.substring(t + 1);
                hash = parseQuery(fragment);
                url = url.substring(0, t);
                modalUrl = hash[Q_PARENT] || EMPTY_STRING;
            }

            return {
                url: modalUrl ? modalUrl : url,
                modalUrl: modalUrl ? url : modalUrl
            };
        }

    });


    var reverseMethodMap = {
        'POST': 'create',
        'GET': 'read'
    };

    function addIframeTransport(form) {
        var dfd = new $.Deferred();
        //generate a random id
        var iframeId = 'unique' + (new Date().getTime());
        var iframe = $('<iframe src="javascript:false;" name="' + iframeId + '" />');

        iframe.hide();
        iframe.appendTo('body');
        iframe.load(function() {
            var loc = iframe[0].contentWindow.location;
            var url = loc.pathname + loc.search + loc.hash;
            var doc = getDoc(iframe[0]);
            var data = getDocumentHTML(doc);
            iframe.remove();
            iframe = null;
            dfd.resolve({
                url: url,
                data: data
            });
        });
        iframe.error(function(ev) {
            iframe.remove();
            iframe = null;
            dfd.reject({
                error: '' + ev
            });
        });
        $(form).attr('target', iframeId);


        return dfd.promise();
    }


    var Page = App.Page = Backbone.Model.extend({

        defaults: {
            url: EMPTY_STRING,
            html: EMPTY_STRING,
            refreshUrl: EMPTY_STRING,
            refreshInterval: null,
            'main-header': null,
            'main-menu': null,
            'main-content': null,
            'site-caption': null,
            'status': null,
            'logs': null
        },

        initialize: function initialize(attributes, options) {
            if (!this.get('url')) {
                throw "Url is required";
            }
            if (this.has('html')) {
                this.set(this.parse(this.get('html'), options));
            }
        },

        clear: function() {
            _.forEach(this.attributes, function(v) {
                if (v instanceof Backbone.Model) {
                    v.clear();
                }
            });
            return Page.__super__.clear.apply(this, arguments);
        },

        toJSON: function() {
            return _.mapValues(this.attributes, function(v) {
                return v instanceof Backbone.Model ? v.toJSON() : _.clone(v);
            });
        },

        url: function() {
            return this.get('url');
        },

        content: function() {
            return $($.parseHTML(this.get('html')));
        },

        refresh: function(options) {
            var url = this.get('refreshUrl') || this.get('url');
            this.set('url', url);
            return this.fetch(options);
        },

        thirdLevelCaption: function() {
            return this.content().find('.top-active-title').last().text();
        },

        submitForm: function(form, action, options) {
            options = options ? _.clone(options) : {};
            if (options.parse === void 0) options.parse = true;
            var model = this;
            var success = options.success;
            var formData = new FormData(form);
            // if (IE.isTheBrowser) {
            formData.append(action.name, action.value);
            // }
            var method = ($(form).attr("method") || 'GET').toUpperCase();
            var url = ($(form).attr('action') || this.url());

            options = _.extend(options, {
                url: url,
                data: formData,
                contentType: false,
                processData: false,
                success: function(resp) {
                    if (!model.set(model.parse(resp, options), options)) return false;
                    if (success) success(model, resp, options);
                    model.trigger('sync', model, resp, options);
                }

            });
            wrapError(this, options);
            return this.sync(reverseMethodMap[method], this, options);
        },

        sync: function(method, model, options) {
            return Page.__super__.sync.apply(this, [method, this, _.extend(options, {
                dataType: 'html'
            })]);
        },

        parse: function(response /*, options*/ ) {
            var $html = $($.parseHTML(response)),
                data = {
                    html: response
                };
            _.extend(data,
                this._parseRefresh($html),
                this._parseLogs($html),
                this._parseMainHeader($html),
                this._parseMainMenu($html),
                this._parseStatus($html),
                this._parseMainContent($html),
                this._parseSiteCaption($html),
                this._parseCopyright($html),
                this._parseBreadcrumbs($html)
            );
            // перенос заголовка из меню в заголовок страницы
            if (data['main-menu'] && data['main-menu'].get('title')) {
                var title = data['main-menu'].get('title');
                data['main-menu'].set('title', '');
                if (data['main-header']) {
                    data['main-header'].set('text', title);
                }
            }
            return data;
        },

        _parseRefresh: function($html) {
            // parse <noscript><meta http-equiv="Refresh" content="10; URL=StatList.html?" /></noscript>
            // the content of <noscript> is a #text
            var meta = $html.filter('#meta_refresh').text();

            if (!meta) {
                return {
                    refreshUrl: null,
                    refreshInterval: null
                };
            }
            var matches = meta.match(/content="([^;]+);\s+URL=([^"]+)"/),
                interval = parseFloat(matches[1]),
                url = matches[2];

            return {
                refreshUrl: cgUrl.urlJoin(this.get('url'), url),
                refreshInterval: interval
            };
        },

        _parseMainHeader: function($html) {
            var $activeTitle, sectionType, pageUrl = this.url();

            $activeTitle = $html.find('.main-menu_main .tabActive')
                .closest('.main-menu_main > .main-menu__item')
                .children('.main-menu__item-title');

            sectionType = parseSectionType(pageUrl);

            return {
                'main-header': new Backbone.Model({
                    text: $activeTitle.text(),
                    sectionType: sectionType
                })
            };
        },

        _parseMainMenu: function($html) {
            var menus = [],
                menu = this._buildMenu($html.find('table:first'), menus);

            return {
                'main-menu': new Backbone.Model(menus.length ? menus[0] : menu)
            };
        },

        _parseMainContent: function($html) {
            var $table = $html.find('form:first').closest('table');

            return {
                'main-content': new Backbone.Model({
                    html: outerHtml($table)
                })
            };
        },

        _parseSiteCaption: function($html) {
            return {
                'site-caption': $html.find('.tabName:first > span').text()
            };
        },

        _parseStatus: function($html) {
            var $statusList = $html.find('.successText, .failureText'),
                $status = $statusList.first(),
                message = $status.text(),
                status = $status.attr('class');

            $statusList.remove();
            return {
                'status': new Backbone.Model({
                    message: message,
                    status: status
                })
            };
        },

        _parseCopyright: function($html) {
            var $copyright = $html.filter('.copyright-container'),
                html = outerHtml($copyright);

            $copyright.remove();
            return {
                'copyright': new Backbone.Model({
                    html: html,
                })
            };
        },

        _parseBreadcrumbs: function($html) {
            var menuItems, items, captions = [];
            menuItems = $html.find('.main-menu__item_opened')
                .add($html.find('.tabActive, .tab:first', 'table:first'))
                .filter(function(i, elem) {
                    var $elem = $(elem);
                    if ($elem.is('.tab') && $elem.siblings('.tabActive').length > 0) {
                        return false;
                    }
                    return elem;
                });
            items = _.map(menuItems, function(elem) {
                var $that = $(elem),
                    $link = $that.find('>a'),
                    $activeTitle = $that.find('.top-active-title'),
                    title, href, hrefFirst, item, url, caption;

                if ($link.length) {
                    title = trim($link.text());
                    hrefFirst = $that.prevAll('.tab:last').andSelf().eq(0).find('a').attr('rel');
                    href = $link.attr('rel') || hrefFirst;
                    url = href ? this.absoluteUrl(href) : '';
                    item = {
                        title: title,
                        url: url,
                        rel: url
                    };
                    if (captions.length && hrefFirst) {
                        caption = captions.pop();
                        caption.url = this.absoluteUrl(hrefFirst);
                        caption.rel = caption.url;
                    }
                } else if ($activeTitle.length) {
                    title = $activeTitle.text();
                    item = {
                        title: title,
                        url: null // ссылкой на заголовке будет первый пункт его подменю
                    };
                    captions.push(item);
                } else {

                }
                return item;
            }, this);

            if (items.length) {
                items[items.length - 1].url = '';
            }

            return {
                'breadcrumbs': new Backbone.Model({
                    items: items
                })
            };
        },

        _parseLogs: function($html) {
            var $logs = $html.filter('.logViewInline'),
                logs = $logs.text();

            $logs.remove();
            return {
                'logs': new Backbone.Model({
                    logs: logs
                })
            };
        },

        _buildMenu: function($table, menus) {
            var that = this,
                menu = null;
            var $rows = $table.find('>tbody>tr:first');

            var $td = $rows.find('>td'),
                $menu = $td.eq(0),
                $data = $td.eq(1),
                items = [],
                $title,
                submenu;

            if ($menu.children().is('table.third_level') &&
                $data.hasClass('tabDataOuter') &&
                $data.children().is('table.tabData')) {

                $title = $menu.find('.top-active');
                submenu = this._buildMenu($data.children(), menus);

                if ($title.length) {
                    menu = {
                        title: $title.text(),
                        items: submenu ? submenu.items : []
                    };
                    if (menus) {
                        menus.push(menu);
                    }
                } else {
                    $menu.find('.tab, .tabActive').each(function() {
                        var title, href, isActive, item;

                        title = trim($(this).text());
                        if (title) {
                            href = $(':link', this).attr('rel');
                            isActive = $(this).hasClass('tabActive');
                            item = {
                                url: href ? that.absoluteUrl(href) : (isActive && submenu && submenu.items.length ? submenu.items[0].url : that.url()),
                                title: title,
                                menu: isActive ? submenu : null,
                                isActive: !!(isActive && submenu),
                                isCurrent: !!(isActive && !submenu)
                            };
                            items.push(item);
                        }
                    });

                    // меню без пунктов или с одним пунктом это не меню, оставляем только где два и больше
                    if (items.length >= 2) {
                        menu = {
                            title: null,
                            items: items
                        };
                    }
                }
            }
            return menu;
        },

        absoluteUrl: function(href) {
            return cgUrl.urlJoin(this.url(), href);
        }

    }, {
        load: function(url) {
            var page = new Page({
                url: url
            });
            return page.fetch().then(function() {
                return page;
            });
        }
    });


    var ProgressView = App.ProgressView = Backbone.View.extend({

        className: 'progress',

        initialize: function() {
            this.speed = 2000;
            this.$bar = null;
            _.bindAll(this, '_rotate');
        },

        render: function() {
            this.$bar = $($.parseHTML('<div class="progress__bar">'))
                .hide();
            this.$el.html(this.$bar);
            setTimeout(this._rotate, 0); // add chance to render bar and calculate width
            return ProgressView.__super__.render.apply(this, arguments);
        },

        remove: function() {
            if (this.$bar) {
                this.$bar.stop();
                this.$bar = null;
            }
            return ProgressView.__super__.remove.apply(this, arguments);
        },

        _rotate: function() {
            if (!this.$bar) {
                return;
            }
            this.$bar
                .css("left", -(this.$bar.width() * 0.95) + 'px')
                .show()
                .animate({
                        left: this.$el.width()
                    },
                    this.speed,
                    this._rotate
                );
        }

    });


    var LogsView = App.LogsView = Backbone.View.extend({

        template: 'logs.html',

        render: function() {
            this.$el.html(render(this.template, this.model.toJSON()));
            return LogsView.__super__.render.apply(this, arguments);
        }

    });

    var ModalView = App.ModalView = Backbone.View.extend({

        el: '.modal',

        events: {
            'click .close': 'onClose'
        },

        initialize: function() {
            _.bindAll(this, 'onBodyClick');
            return ModalView.__super__.initialize.apply(this, arguments);
        },

        render: function() {
            this.$el.html(render('layer.html', {}));
            return this;
        },

        dialog: function() {
            this.dfd = new $.Deferred();
            if (this.subView) {
                this.subView.remove();
                this.subView = null;
            }
            return this.dfd;
        },

        show: function(view) {
            if (this.subView) {
                this.subView.remove();
            }
            this.subView = view.render();
            this.$('.layer-body').remove();
            this.$('.layer-head').after(this.subView.$el);
            if (this.$('.layer-body .menu').length) {
                this.$('.layer-body').addClass('layer-body_has-menu');
            }

            $('body').addClass('modal-open');
            $('body').on('click', this.onBodyClick);
            this.$el.show();
            this.forceReflow();
            this.$el.addClass('in');
            $('select, input', this.$el).styler(STYLER_OPTIONS);
        },

        hide: function() {
            var dfd = new $.Deferred();

            $('body').off('click', this.onBodyClick);
            if (!$('body').hasClass('modal-open')) {
                dfd.resolve();
            } else {
                $('body').removeClass('modal-open');
                this.$el.removeClass('in');
                this.forceReflow();
                setTimeout(function() {
                    this.$el.hide();
                    if (this.view) {
                        this.view.remove();
                        this.view = null;
                    }
                    dfd.resolve();
                }.bind(this), 300);
            }

            return dfd.promise();
        },

        setTitle: function(value) {
            this.$('.layer-head p').text(value);
        },

        onClose: function(ev) {
            ev.preventDefault();
            this.hide().then(function() {
                this.dfd.resolve();
            }.bind(this));
        },

        onBodyClick: function(ev) {
            if ($(ev.target).parent().closest(this.$el).length === 0) {
                ev.preventDefault();
                this.hide().then(function() {
                    this.dfd.resolve();
                }.bind(this));
            }
        },

        forceReflow: function(el) {
            (el && el[0] || this.$el[0]).offsetWidth; // jshint ignore:line
        }

    });


    var MenuView = App.MenuView = Backbone.View.extend({

        events: {
            '.menu__item click': 'onClick'
        },

        template: 'menu.html',

        initialize: function(options) {
            this.mods = options.mods || {};
            MenuView.__super__.initialize.apply(this, arguments);
        },

        render: function() {
            this.$el.html(render(this.template, {
                menu: this.model.toJSON(),
                mods: this.mods
            }));
            return MenuView.__super__.render.apply(this, arguments);
        },

        onClick: function() {
            this.$('.menu__item.active').not(this).removeClass('active');
            $(this).addClass('active');
        }
    });


    var MainHeaderView = App.MainHeaderView = Backbone.View.extend({

        template: 'header.html',

        render: function() {
            this.$el.html(render(this.template, {
                header: this.model.toJSON()
            }));
            return MainHeaderView.__super__.render.apply(this, arguments);
        }

    });


    var HtmlFragmentView = App.HtmlFragmentView = Backbone.View.extend({

        render: function() {
            this.$el.html(this.model.get('html'));
            return HtmlFragmentView.__super__.render.apply(this, arguments);
        }

    });

    var StatusView = App.StatusView = Backbone.View.extend({

        events: {
            'click .status__close': 'onClose'
        },

        template: 'status.html',

        render: function() {
            var data = this.model.toJSON();
            if (data.message) {
                this.$el.html(render(this.template, data));
                this.$el.fadeIn();
                this.$el.css('display', 'inline-block');
            } else {
                this.close();
            }
            return StatusView.__super__.render.apply(this, arguments);
        },

        onClose: function() {
            this.close();
        },

        close: function() {
            this.$el.hide();
        }

    });


    var SubmitableView = App.SubmitableView = Backbone.View.extend({

        events: {
            'submit form': 'onSubmit'
        },

        initialize: function() {
            this.ui = {};
            return SubmitableView.__super__.initialize.apply(this, arguments);
        },

        onSubmit: function(ev) {
            var view = this,
                model = this.model,
                form = ev.target,
                $btn = $(':focus', ev.target),
                action = $btn.is(':submit') ? {
                    name: $btn.attr('name'),
                    value: $btn.attr('value')
                } : undefined;

            addIframeTransport(form).then(function(resp) {
                model.set('url', resp.url);
                model.set(model.parse(resp.data));
                model.trigger('sync', model, resp.data);
                view.trigger('submit', {
                    page: model,
                    action: action
                });
            });
        },

        remove: function() {
            this.removeAllViews();
            return SubmitableView.__super__.remove.apply(this, arguments);
        },

        addView: function(name, view) {
            this.ui[name] = view.render();
        },

        removeAllViews: function() {
            _.forEach(this.ui, function(view, name) {
                view.remove();
                delete this.ui[name];
            }, this);
        }
    });


    var PageView = App.PageView = SubmitableView.extend({

        template: 'main.html',

        render: function() {
            this.$el.html(render(this.template));

            this.addView('header', new MainHeaderView({
                el: this.$('.main__header'),
                model: this.model.get('main-header')
            }));

            this.addView('menu', new MenuView({
                el: this.$('.main__menu'),
                mods: 'menu_theme_content',
                model: this.model.get('main-menu')
            }));

            this.addView('content', new HtmlFragmentView({
                el: this.$('.main__content'),
                model: this.model.get('main-content')
            }));

            this.addView('logs', new LogsView({
                el: this.$('.main__logs'),
                model: this.model.get('logs')
            }));

            // показывать или в блоке кнопок "Модифицировать", или вверху экрана
            var $statusElem = this.$('.common-btns-line .success-text');
            if ($statusElem.length === 0) {
                $statusElem = this.$('.main__status');
            }
            this.addView('status', new StatusView({
                el: $statusElem,
                model: this.model.get('status')
            }));

            this.addView('breadcrumbs', new BreadcrumbsView({
                el: this.$('.main__breadcrumbs'),
                model: this.model.get('breadcrumbs')
            }));
            this.addView('copyright', new HtmlFragmentView({
                el: this.$('.main__copyright'),
                model: this.model.get('copyright')
            }));

            if (this.$('.main__menu .menu').length) {
                this.$('.main').addClass('main_has-menu');
            }

            if (!this.model.url().match(/Settings\/TriggerEvents/)) {
                $('select, input', this.$el).styler(STYLER_OPTIONS);
            }
            return this;
        }

    });


    var ModalPageView = App.ModalPageView = SubmitableView.extend({

        className: 'layer-body',

        render: function() {
            var content = this.model.get('main-content');
            this.$el.html(content.get('html'));
            return this;
        }
    });


    var MainMenuView = App.MainMenuView = Backbone.View.extend({

        events: {
            'click .main-menu__opener': 'onOpenerClick',
            'click .main-menu__item-title': 'onClickTitle'
        },

        initialize: function() {
            this._cookieName = 'main-menu';
            this._opened = [];
            this.$menu = null;
            this.$items = null;
            _.bindAll(this, 'openItem', '_restoreItem');
        },

        render: function() {
            var that = this;
            this.$menu = this.$el.find('.main-menu_main');
            if (this.inDomainAdmin(this.getBasePath())) {
                this.$menu.addClass('main-menu_user-domain');
            }
            this.$items = this.$menu.children('.main-menu__item');
            this._adjustMenuItemLinks(this.getBasePath(), this.$menu);
            this._restore().then(function() {
                setTimeout(function() {
                    that.$menu.show();
                }, 0);
            });
            return MainMenuView.__super__.render.apply(this, arguments);
        },

        remove: function() {
            this.$menu = null;
            this.$items = null;
            return MainMenuView.__super__.remove.apply(this, arguments);
        },

        getBasePath: function() {
            var pathname = window.location.pathname,
                basePath;

            basePath = pathname.replace(/[^\/]*$/, '');
            return basePath;
        },

        onOpenerClick: function(ev) {
            var $elem = $(ev.target);
            ev.preventDefault();
            this.toggleItem($elem.closest('.main-menu__item'));
        },

        onClickTitle: function(ev) {
            var $elem = $(ev.target),
                $item = $elem.closest('.main-menu__item');
            this.selectItem($item);
        },

        selectItemByPath: function(breadcrumbs) {
            // т.к. ссылки могут отличатся ищем по заголовкам меню.
            var $link, $first, path;

            path = this.inDomainAdmin(this.getBasePath()) ?
                breadcrumbs.slice(0, 1) : breadcrumbs.slice(0, 2);

            if (path.length === 2) {
                $first = this.$menu
                    .find('>.main-menu__item > .main-menu__item-title')
                    .filter(function() {
                        return trim($(this).text()) === path[0].title;
                    })
                    .first();
                $link = $first.parent()
                    .find('.main-menu > .main-menu__item > .main-menu__item-title')
                    .filter(function() {
                        return trim($(this).text()) === path[1].title;
                    })
                    .first();
            } else {
                $link = this.$menu.find("[href^='" + path[0].rel + "']");
            }

            this.selectItem($link.parent());
        },

        selectItem: function($item) {
            var $menu = $item.closest('.main-menu'),
                activeCls = 'tabActive',
                $upperItem = $menu.closest('.main-menu__item');

            $('.main-menu__item', this.$menu).not($item).removeClass(activeCls);
            $item.addClass(activeCls);

            if ($menu.hasClass('main-menu_main') && $item.children('.main-menu__opener').length) {
                this.openItem($item);
                $item.find('.main-menu .main-menu__item:first').addClass(activeCls);
            }
            if ($upperItem.length && !this.isOpenedItem($upperItem)) {
                this.openItem($upperItem);
            }
        },

        isOpenedItem: function(item) {
            var itemId = this._getItemId(item);
            return _.indexOf(this._opened, itemId) !== -1;
        },

        toggleItem: function(item) {
            if (this.isOpenedItem(item)) {
                this.closeItem(item);
            } else {
                this.openItem(item);
            }
        },

        loadItem: function($item) {
            var dfd;

            if ($item.has('.main-menu_primary').length === 0) {
                dfd = this._loadPrimaryMenu($item);
            } else {
                dfd = $.Deferred().resolve($item).promise();
            }
            return dfd;
        },

        openItem: function($item, preventAnimation) {
            if (!$item.hasClass('main-menu__item_opened')) {
                $item.addClass('main-menu__item_opened');
                if (!!preventAnimation) {
                    $item.find('.main-menu_primary').show();
                } else {
                    $item.find('.main-menu_primary').slideDown("fast");
                }
            }

            if (!this.isOpenedItem($item)) {
                this._opened.push(this._getItemId($item));
                this._save();
            }

            return $item;
        },

        closeItem: function(item) {
            var itemId = this._getItemId(item);

            item.removeClass('main-menu__item_opened');
            item.find('.main-menu_primary').slideUp('fast');

            if (this.isOpenedItem(item)) {
                this._opened.splice(_.indexOf(this._opened, itemId), 1);
                this._save();
            }
        },

        getItemLink: function(item) {
            return item.children('.main-menu__item-title');
        },

        inDomainAdmin: function(url) {
            return url.match(/\/DomainAdmin\//);
        },

        _restore: function() {
            var openedBefore = ($.cookie(this._cookieName) || '').split(','),
                currentItem = this.$items.filter('.main-menu__item_opened'),
                currentItemId = this._getItemId(currentItem),
                i;

            this._opened.length = 0;
            for (i = openedBefore.length - 1; i >= 0; i--) {
                var t = parseInt(openedBefore[i], 10);
                if (t >= 0) {
                    this._opened.push(t);
                }
            }

            // append current item to opened
            if (currentItemId !== -1 && _.indexOf(this._opened, currentItemId) === -1) {
                this._opened.push(currentItemId);
                this._save();
            }

            var promises = [];
            for (i = this.$items.length - 1; i >= 0; i--) {
                promises.push(this.loadItem(this.$items.eq(i)).then(this._restoreItem));
            }
            return $.when(promises);
        },

        _restoreItem: function($item) {
            var idx = this.$items.index($item);
            if (_.indexOf(this._opened, idx) !== -1) {
                this.openItem($item, true);
            }
        },

        _save: function() {
            $.cookie(this._cookieName, this._opened.join(','), {
                path: '/'
            });
        },

        _getItemId: function(item) {
            return this.$items.index(item);
        },

        _appendPrimaryMenu: function(item, primaryMenu) {
            var link = this.getItemLink(item),
                url = link.attr('href');

            $('.main-menu__item', primaryMenu)
                .removeClass('tabActive')
                .removeClass('main-menu__item_opened');
            this._adjustMenuItemLinks(url, primaryMenu);
            primaryMenu.hide();
            item.append(primaryMenu);
        },

        _adjustMenuItemLinks: function(baseUrl, primaryMenu) {
            primaryMenu.find('.main-menu__item-title').each(function() {
                var $link = $(this),
                    url = cgUrl.urlJoin(baseUrl, $link.attr('rel')),
                    sectionType;

                $link.attr('href', url);

                if ($link.closest('.main-menu').hasClass('main-menu_main')) {
                    sectionType = parseSectionType(url);
                    $link.addClass(sectionType);
                }
            });
        },

        _loadPrimaryMenu: function(item) {
            var that = this,
                $link = that.getItemLink(item),
                url = $link.attr('href');

            if (!url) return (new $.Deferred()).reject(item).promise();

            return $.ajax({
                url: url,
                dataType: 'html',
            }).then(function(html) {
                $(html).find('.main-menu_primary').each(function() {
                    that._appendPrimaryMenu(item, $(this));
                });
                return item;
            });
        }
    });


    var MainView = App.MainView = Backbone.View.extend({

        el: document,

        subView: undefined,

        events: {
            "click a": "hijackLinks"
        },

        hijackLinks: function(ev) {
            // Get the absolute anchor href.
            var $link = $(ev.currentTarget);
            if ($link.attr('target')) {
                return;
            }
            var href = {
                prop: $link.prop("href"),
                attr: $link.attr("href")
            };
            var root = window.location.protocol + "//" + window.location.host + App.root;

            if (href.prop.slice(0, root.length) === root) {
                var isPageUrl = href.attr.indexOf('.html') !== -1;

                if (isPageUrl) {
                    ev.preventDefault();

                    var url = App.ui.absoluteUrl(href.attr);
                    var inModal = url.match(/Monitors\/SNMPElement.html/);

                    if (inModal) {
                        url = App.router.getModalUrl(window.location.pathname + window.location.search, url);
                    }

                    App.router.navigate(url, true);
                }
            }
        },

        show: function(view) {
            if (this.subView) {
                this.subView.remove();
                this.subView = null;
            }
            this.subView = view.render();
            this.$('.main_container').html(this.subView.$el);
        }
    });


    var BreadcrumbsView = App.BreadcrumbsView = Backbone.View.extend({
        template: 'breadcrumbs.html',

        render: function() {
            var data = this.model.toJSON();
            this.$el.html(render(this.template, data));
            return BreadcrumbsView.__super__.render.apply(this, arguments);
        }

    });

    var ViewManager = App.ViewManager = App.Controller.extend({

        initialize: function() {
            this.mainView = new MainView({
                el: document
            });
            this.modalView = new ModalView({
                el: '.modal'
            });
            this.leftMenu = new MainMenuView({
                el: '.left_column_inner'
            });
            this.leftMenu.render();
            this.progressBar = new ProgressBar();
            this.modalView.render();
            this.page = null;
            this.mainPage = null;
            this.refreshInterval = null;
            this.deferredShowPage = this.buildDeferredView(this.showPage);
            this.deferredShowModalPage = this.buildDeferredView(this.showModalPage);
        },

        showPage: function showPage(page) {
            this.page = page;
            this.mainPage = page;

            return this.modalView.hide().then(function() {
                var view;

                view = new PageView({
                    model: this.page
                });
                view.on('submit', function(ev) {
                    this.deferredShowPage(this.submit(ev.page, ev.action));
                }.bind(this));

                this.mainView.show(view);

                var path;
                if (this.page.get('breadcrumbs')) {
                    path = this.page.get('breadcrumbs').get('items');
                }
                this.leftMenu.selectItemByPath(path.length ? path : [{
                    url: this.page.url()
                }]);

                this.startViewRefresh(this.page, this.deferredShowPage);
            }.bind(this));

        },

        showModalPage: function showModalPage(page) {
            var that = this,
                view;

            this.page = page;

            this.modalView.dialog().then(function() {
                that.mainPage.set('url', ''); // XXX: force reload mainPage
                App.router.navigate(App.router.parseModalUrl(window.location.pathname + window.location.search + window.location.hash).url, true);
            });

            view = new ModalPageView({
                model: page
            });

            view.on('submit', function(ev) {
                this.deferredShowModalPage(this.submit(ev.page, ev.action));
            }.bind(this));

            this.modalView.setTitle(page.thirdLevelCaption() || page.get('site-caption'));
            this.modalView.show(view);
            this.startViewRefresh(page, this.deferredShowModalPage);
        },

        startViewRefresh: function startViewRefresh(page, view) {
            if (this.refreshInterval) {
                clearTimeout(this.refreshInterval);
                this.refreshInterval = null;
            }
            if (page.has('refreshInterval')) {
                this.refreshInterval = setTimeout(
                    function() {
                        view(page.refresh().then(function() {
                            return page;
                        }));
                    },
                    page.get('refreshInterval') * 1000
                );
            }
        },

        getPageUrl: function getPageUrl(page) {
            return (page && page.get('url')) || App.root;
        },

        submit: function(page, action) {
            var dfd = new $.Deferred();

            if (page.url().match(/Directory\/Browser.html/) &&
                // HACK: навигация в "Browser" работает через POST запросы,
                // причем так, что в ответе возвращается контент, но URL остается прежним.
                // Нам же надо обновлять URL в адресной строке, иначе наша чудесная навигация ломается.
                action && _.indexOf(['Up', 'Display'], action.name) !== -1) {
                var baseDN = page.content().find('[name=NewBaseDN]').val(),
                    pageUrl = page.url().replace(/\?.*$/, ''),
                    query = baseDN ? '?baseDN=' + encodeURIComponent(baseDN) : '',
                    url = query ? pageUrl + query : pageUrl;

                App.router.navigate(url, true);
                dfd.reject();
            } else {
                App.router.navigate(page.url(), false);
                dfd.resolve(page);
            }

            return dfd.promise();
        },

        absoluteUrl: function(href) {
            return cgUrl.urlJoin(this.getPageUrl(this.page), href);
        },

        buildDeferredView: function buildDeferredView(view) {
            var ui = this;
            return function(dfd) {
                var progressBar = ui.progressBar.show();
                return dfd.then(function(page) {
                    progressBar.hide();
                    return view.call(ui, page);
                }, function(error) {
                    progressBar.hide();
                    if (error) {
                        console.error(error);
                    }
                });
            };
        },

    });


    /**
     * Позволяет создавать сколько угодно прогресс-баров, которые отображаются одним view
     */
    function ProgressBar() {
        this.barRegistry = [];
        this.delay = 300; // ждать перед показом, milliseconds
        this.view = null;
    }

    ProgressBar.prototype.show = function() {
        var that = this;
        var bar = {
            hide: function() {
                that.hide(this);
            }
        };
        if (this.barRegistry.length === 0) {
            this.view = new ProgressView();
            setTimeout(function() {
                if (this.view) {
                    $('body').append(this.view.render().$el);
                }
            }.bind(this), this.delay);
        }
        this.barRegistry.push(bar);
        return bar;
    };

    ProgressBar.prototype.hide = function(bar) {
        var idx = _.indexOf(this.barRegistry, bar);
        if (idx > -1) {
            this.barRegistry.splice(idx, 1);
        } else {
            throw "Progress not exists";
        }
        if (this.barRegistry.length === 0) {
            this.view.remove();
            this.view = null;
        }
    };

    ProgressBar.prototype.hideAll = function() {
        while (this.barRegistry.length) {
            this.hide(this.barRegistry[0]);
        }
    };


    function parseQuery(query) {
        var match,
            pl = /\+/g, // Regex for replacing addition symbol with a space
            search = /([^&=]+)=?([^&]*)/g,
            decode = function(s) {
                return decodeURIComponent(s.replace(pl, " "));
            },
            urlParams = {};

        while (match = search.exec(query)) { // jshint ignore:line
            urlParams[decode(match[1])] = decode(match[2]);
        }
        return urlParams;
    }

    function getDocumentHTML(doc) {
        var node = doc.doctype;
        return node ? [
            '<!DOCTYPE ',
            '', node.name,
            '', (node.publicId ? ' PUBLIC "' + node.publicId + '"' : EMPTY_STRING),
            '', (!node.publicId && node.systemId ? ' SYSTEM' : EMPTY_STRING),
            '', (node.systemId ? ' "' + node.systemId + '"' : EMPTY_STRING),
            '>\n',
            doc.documentElement.outerHTML
        ].join('') : doc.documentElement.outerHTML;
    }

    function outerHtml($el) {
        return $("<div />").append($($el).clone()).html();
    }

    // from backbone.js
    // Wrap an optional error callback with a fallback error event.
    var wrapError = function(model, options) {
        var error = options.error;
        options.error = function(resp) {
            if (error) error(model, resp, options);
            model.trigger('error', model, resp, options);
        };
    };


    function trim(str) {
        return str.replace(/^\s+/, '').replace(/\s+$/, '');
    }

    function getDoc(iframe) {
        var doc = null;

        // IE8 cascading access check
        try {
            if (iframe.contentWindow) {
                doc = iframe.contentWindow.document;
            }
        } catch (err) {}

        if (doc) { // successful getting content
            return doc;
        }

        try { // simply checking may throw in ie8 under ssl or mismatched protocol
            doc = iframe.contentDocument ? iframe.contentDocument : iframe.document;
        } catch (err) {
            // last attempt
            doc = iframe.document;
        }
        return doc;
    }

    function inIframe() {
        try {
            return window.self !== window.top;
        } catch (e) {
            return true;
        }
    }

    function parseSectionType(url) {
        var sectionType;
        if (url.match(/\/DomainAdmin\//)) {
            sectionType = url.replace(/\.html.*$/, '').replace(/^.+\//, '').toLowerCase();
        } else {
            sectionType = url.replace(/\.html.*$/, '').split('/').slice(-2)[0].toLowerCase();
        }

        return sectionType;
    }

    $(function() {
        if (!inIframe()) {
            $('.settingsBox caption').on('click', function() {
                $(this).next('tbody').toggle();
            });

            $('.left_column_inner').jScrollPane({
                autoReinitialise: true,
                maintainPosition: false
            });

            window.app = App.bootstrap();
        }
    });

}(jQuery, _, Backbone));
